# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class ModuleDocFragment(object):
    DOCUMENTATION = r'''
options:
  cluster:
    description:
      - The name of the cluster where the row policy is defined. If not provided, the row policy will be managed on the local server.
    type: str
    default: null
'''
